package com.baofoo.rsa;


import org.apache.commons.lang3.ArrayUtils;

import com.baofoo.util.SecurityUtil;

import javax.crypto.Cipher;
import java.security.*;
import java.security.spec.RSAPrivateKeySpec;
import java.security.spec.RSAPublicKeySpec;


public class JuheRSA {
    private static final String ALGORITHM_NAME = "RSA";

    /**
     * 公钥加密
     * @param data
     *          源报文字节数组
     * @param base64CertText
     *          公钥字符串
     * @return
     *      16进制字符串
     */
    
    public static String encrypt(byte[] data, String CertPath){
    	try {
    		return SecurityUtil.byte2Hex(encrypt0(RsaReadUtil.getPublicKeyFromFile(CertPath), data));
    	}catch(Exception ex) {
    		ex.printStackTrace();
    		return "";
    	}
        
    }

 

    /**
     * 私钥解密
     * @param data
     *          加密字符串
     * @param base64PfxText
     *          私钥 base64 字符串
     * @param password
     *          私钥密码
     * @return
     *      16进制字符串
     */
    public static byte[] decrypt(String data, String PfxPath, String password) {
    	try {    		
    		return decrypt0(RsaReadUtil.getPrivateKeyFromFile(PfxPath, password), SecurityUtil.hex2Bytes(data));
    	}catch(Exception ex) {
    		ex.printStackTrace();
    		return null;
    	}
        
    }


    public static byte[] encrypt0(PublicKey publicKey, byte[] srcData) throws Exception{
        Cipher cipher = configureCipher(publicKey, Cipher.ENCRYPT_MODE);
        return doFinal(acquirePublicKeyLength(publicKey), Cipher.ENCRYPT_MODE, srcData, cipher);
    }

    public static byte[] decrypt0(PrivateKey privateKey, byte[] srcData) throws Exception{
        Cipher cipher = configureCipher(privateKey, Cipher.DECRYPT_MODE);
        return doFinal(acquirePrivateKeyLength(privateKey), Cipher.DECRYPT_MODE, srcData, cipher);
    }

    private static Cipher configureCipher(Key key, int mode) throws Exception{
        try {
            Cipher cipher = Cipher.getInstance(ALGORITHM_NAME);
            cipher.init(mode, key);
            return cipher;
        } catch (GeneralSecurityException e) {
            throw new Exception("配置Cipher实例异常", e);
        }
    }

    private static int acquirePublicKeyLength(PublicKey key) throws Exception{
        try {
            KeyFactory keyFact = KeyFactory.getInstance(key.getAlgorithm());
            RSAPublicKeySpec keySpec = keyFact.getKeySpec(key, RSAPublicKeySpec.class);
            return keySpec.getModulus().toString(2).length();
        } catch (GeneralSecurityException e) {
            throw new Exception("获取公钥长度异常", e);
        }
    }

    private static int acquirePrivateKeyLength(PrivateKey key) throws Exception{
        try {
            KeyFactory keyFact = KeyFactory.getInstance(key.getAlgorithm());
            RSAPrivateKeySpec keySpec = keyFact.getKeySpec(key, RSAPrivateKeySpec.class);
            return keySpec.getModulus().toString(2).length();
        } catch (GeneralSecurityException e) {
            throw new Exception("获取私钥长度异常", e);
        }
    }

    private static byte[] doFinal(int length, int mode, byte[] srcData, Cipher cipher) throws Exception{
        try {
            int blockSize = mode == 1 ? 117 : 128;
            if (2048 == length) {
                blockSize = mode == 1 ? 245 : 256;
            }
            byte[] data = null;
            for(int i = 0; i < srcData.length; i += blockSize) {
                byte[] doFinal = cipher.doFinal(ArrayUtils.subarray(srcData, i, i + blockSize));
                data = ArrayUtils.addAll(data, doFinal);
            }
            return data;
        } catch (GeneralSecurityException e) {
            throw new Exception("cipher doFinal异常", e);
        }
    }


}
