package com.baofoo.util;

import javax.crypto.*;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.io.UnsupportedEncodingException;
import java.security.*;

public class TripleDes {

    private static final String ALGORITHM_NAME = "DESede";

    private static final String ALGORITHM_NAME_CBC_PADDING = "desede/CBC/PKCS5Padding";

    private static final int KEY_SIZE = 168;


    /**
     * 生成168bit的随机字节数组
     * @return
     *          byte[]
     * @throws CryptoException
     */
    public static byte[] generateKey0() throws Exception {
        try {
            KeyGenerator kg = KeyGenerator.getInstance(ALGORITHM_NAME);
            kg.init(KEY_SIZE, new SecureRandom());
            return kg.generateKey().getEncoded();
        } catch (NoSuchAlgorithmException e) {
            throw new Exception("生成随机字符串异常", e);
        }
    }

    /**
     * 生成168bit的16进制字符串
     * @return
     *          String 16进制
     * @throws CryptoException
     */
    public static String generateKey() throws Exception{
        return SecurityUtil.byte2Hex(generateKey0());
    }

    /**
     * 配置Cipher实例
     * @param algorithmName
     * @param mode
     * @param key
     * @param iv
     *          初始向量
     * @return
     * @throws CryptoException
     */
    public static Cipher configureCipher(String algorithmName, int mode, byte[] key, byte[] iv) throws Exception{
        try {
            Cipher cipher = Cipher.getInstance(algorithmName);
            cipher.init(mode, new SecretKeySpec(key, ALGORITHM_NAME), new IvParameterSpec(iv));
            return cipher;
        } catch (GeneralSecurityException e) {
            throw new Exception("配置Cipher实例异常", e);
        }
    }

    /**
     * 加密
     * @param key
     *          密钥字节数组
     * @param data
     *          加密数据字节数组
     * @param iv
     *          初始向量
     * @return
     *          加密后字节数组
     * @throws CryptoException
     */
    public static byte[] encryptCbc0(byte[] key, byte[] data, byte[] iv) throws Exception{
        try {
            Cipher cipher = configureCipher(ALGORITHM_NAME_CBC_PADDING, Cipher.ENCRYPT_MODE, key, iv);
            return cipher.doFinal(data);
        } catch (GeneralSecurityException e) {
            throw new Exception("cipher doFinal异常", e);
        }
    }

    /**
     * 加密
     * @param key
     *          密钥字节数组
     * @param data
     *          加密数据字节数组
     * @param iv
     *          初始向量
     * @return
     *          加密后string字符串
     * @throws Exception 
     */
    public static String encryptCbc(byte[] key, byte[] data, byte[] iv) throws Exception{
        return SecurityUtil.byte2Hex(encryptCbc0(key, data, iv));
    }

    /**
     * 加密
     * @param key
     *          16进制密钥字符串
     * @param data
     *          加密数据字符串
     * @param  iv
     *          初始向量
     * @return
     *          加密后string字符串
     * @throws CryptoException
     */
    public static String encryptCbc(String key, String data, byte[] iv) throws Exception{
        try {
        	
            return SecurityUtil.byte2Hex(encryptCbc0(SecurityUtil.hex2Bytes(key), data.getBytes("UTF-8"), iv));
        } catch (UnsupportedEncodingException e) {
            throw new Exception("字符编码异常", e);
        }
    }

    /**
     * 解密
     * @param key
     *          解密密钥字节数组
     * @param data
     *          解密数据字节数组
     * @param iv
     *          初始向量
     * @return
     *          解密后字节数组
     * @throws CryptoException
     */
    public static byte[] decryptCbc0(byte[] key, byte[] data, byte[] iv) throws Exception{
        try {
            Cipher cipher = configureCipher(ALGORITHM_NAME_CBC_PADDING, Cipher.DECRYPT_MODE, key, iv);
            return cipher.doFinal(data);
        } catch (GeneralSecurityException e) {
            throw new Exception("cipher doFinal异常", e);
        }
    }

    /**
     * 解密
     * @param key
     *          解密密钥字节数组
     * @param data
     *          解密数据字节数组
     * @param iv
     *          初始向量
     * @return
     *          byte[]
     * @throws Exception 
     */
    public static byte[] decryptCbc(byte[] key, byte[] data, byte[] iv) throws Exception{
        return decryptCbc0(key, data, iv);
    }

    /**
     * 解密
     * @param key
     *          16进制密钥字符串
     * @param data
     *          解密数据字符串
     * @param iv
     *          初始向量
     * @return
     *          byte[]
     * @throws Exception 
     */
    public static byte[] decryptCbc(String key, String data, byte[] iv) throws Exception{
    	
        return decryptCbc0(SecurityUtil.hex2Bytes(key), SecurityUtil.hex2Bytes(data), iv);
    }
}
